import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import pystan
import seaborn as sb
import pickle


def CalcMAPEst(samples, a = 0.05, n_points = 10000):
    """
        Calculation of the MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    print('Calculating the mode.')
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]       
    return MAP_Est, est_pdf[map_idx]


def calcHDI(sample, a = 0.05):
    """
            Calculation of 100(1-a)% HDI for sample
            Chen-Shao HPD Estimation algorithm
    """
    print('Calculating HDI.')
    smpl = np.sort(sample)
    n = len(smpl)
    ckW = max(smpl) - min(smpl)
    cki = 0
    RB = int(n * (1-a))
    L0 = 0
    U0 = n-1
    while (RB + cki < n):
        if smpl[RB + cki] - smpl[cki] < ckW:
            ckW = smpl[RB + cki] - smpl[cki]
            L0 = cki
            U0 = RB + cki
        cki += 1
    ckW = max(smpl) - min(smpl)
    cki = 1
    LB = n - int(n * (1-a))
    L1 = 0
    U1 = n-1
    while (LB - cki >= 0):
        if smpl[n - cki] - smpl[LB - cki] < ckW:
            ckW = smpl[n - cki] - smpl[LB - cki]
            U1 = n - cki
            L1 = LB - cki
        cki += 1
    return smpl[int((L1+L0)/2)], smpl[int((U1+U0)/2)]



k = int(input('k = '))
N = int(input('N = '))

Data = {'N': N, 'k': k}

sm = pystan.StanModel(file = 'tri_test.stan')
    
fit = sm.sampling(data = Data, iter = 10_000, n_jobs = 1)

print(fit)

sb.kdeplot(fit['Pd'])
plt.hist(fit['Pd'], density = True, color = 'g', alpha = 0.5)

MAPEst = CalcMAPEst(fit['Pd'])[0]
plt.scatter([MAPEst], [0], marker = 'o', s = 200, 
            c = 'magenta', label = f'mode = {MAPEst:.2f}')
L_bndry, R_bndry = calcHDI(fit['Pd'], 0.05)
plt.plot([L_bndry, R_bndry], [0, 0], c = 'r', lw = 5,
         label = '95% HDI [{0:.2f}, {1:.2f}]'.format(L_bndry, R_bndry))
plt.yticks([])
plt.xlabel('Pd', fontsize = 12)
plt.title(f'Posterior distribution\nk = {k}, N = {N}', fontsize = 16)
plt.legend()
plt.savefig('FigPd.png')
plt.show()
