import numpy as np
import scipy.stats as ss
import pystan
import matplotlib.pyplot as plt
import seaborn as sb
import pickle


def CalcMAPEst(samples, a = 0.05, n_points = 10000):
    """
        Calculation of the MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    print('Calculating the mode.')
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]       
    return MAP_Est, est_pdf[map_idx]


def calcHDI(sample, a = 0.05):
    """
            Calculation of 100(1-a)% HDI for sample
            Chen-Shao HPD Estimation algorithm
    """
    print('Calculating the HDI.')
    smpl = np.sort(sample)
    n = len(smpl)
    ckW = max(smpl) - min(smpl)
    cki = 0
    RB = int(n * (1-a))
    L0 = 0
    U0 = n-1
    while (RB + cki < n):
        if smpl[RB + cki] - smpl[cki] < ckW:
            ckW = smpl[RB + cki] - smpl[cki]
            L0 = cki
            U0 = RB + cki
        cki += 1
    ckW = max(smpl) - min(smpl)
    cki = 1
    LB = n - int(n * (1-a))
    L1 = 0
    U1 = n-1
    while (LB - cki >= 0):
        if smpl[n - cki] - smpl[LB - cki] < ckW:
            ckW = smpl[n - cki] - smpl[LB - cki]
            U1 = n - cki
            L1 = LB - cki
        cki += 1
    return smpl[int((L1+L0)/2)], smpl[int((U1+U0)/2)]


kB = int(input('kB = '))
NB = int(input('NB = '))
kC = int(input('kC = '))
NC = int(input('NC = '))
kBC = int(input('kBC = '))
NBC = int(input('NBC = '))

sm = pystan.StanModel(file = 'tri_test_logistic.stan')

Data = {'kB': kB, 'NB': NB, 'kC': kC, 'NC': NC, 'kBC': kBC, 'NBC': NBC}
fit = sm.sampling(data = Data, iter = 10_000, n_jobs = 1)
print(fit)
#
#       Posterior distributions of factors fB, fC, and fBC
#
MAPfB = CalcMAPEst(fit['fB'])[0]
MAPfC = CalcMAPEst(fit['fC'])[0]
MAPfBC = CalcMAPEst(fit['fBC'])[0]
L_HDIfB, R_HDIfB = calcHDI(fit['fB'])
L_HDIfC, R_HDIfC = calcHDI(fit['fC'])
L_HDIfBC, R_HDIfBC = calcHDI(fit['fBC'])
sb.kdeplot(fit['fB'], label = 'fB, mode = {0:.2f}, \n    95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPfB, L_HDIfB, R_HDIfB))
sb.kdeplot(fit['fC'], label = 'fC, mode = {0:.2f}, \n    95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPfC, L_HDIfC, R_HDIfC))
sb.kdeplot(fit['fBC'], label = 'fBC, mode = {0:.2f}, \n     95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPfBC, L_HDIfBC, R_HDIfBC))
plt.xlabel('Factor', fontsize = 14)
plt.yticks([])
plt.title('Posterior distribtion\n' + \
          ('kB = {0:}, NB = {1:}, kC = {2:}, NC = {3:}\n'.format(kB, NB, kC, NC)) +\
          ('kBC = {0:}, NBC = {1:}'.format(kBC, NBC)), fontsize = 16)
plt.legend()
plt.tight_layout()
plt.savefig('FigFctrs.png')
plt.show()
#
#        Posterior distributions of PdB, PdC, and PdBC
#
MAPPB = CalcMAPEst(fit['PdB'])[0]
MAPPC = CalcMAPEst(fit['PdC'])[0]
MAPPBC = CalcMAPEst(fit['PdBC'])[0]
L_HDIPB, R_HDIPB = calcHDI(fit['PdB'])
L_HDIPC, R_HDIPC = calcHDI(fit['PdC'])
L_HDIPBC, R_HDIPBC = calcHDI(fit['PdBC'])
sb.kdeplot(fit['PdB'], label = 'PdB, mode = {0:.2f}, \n    95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPPB, L_HDIPB, R_HDIPB))
sb.kdeplot(fit['PdC'], label = 'PdC, mode = {0:.2f}, \n    95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPPC, L_HDIPC, R_HDIPC))
sb.kdeplot(fit['PdBC'], label = 'PdBC, mode = {0:.2f}, \n     95% HDI = [{1:.2f}, {2:.2f}]'.
           format(MAPPBC, L_HDIPBC, R_HDIPBC))
plt.xlabel('Prob. of detection', fontsize = 14)
plt.yticks([])
plt.title('Posterior distribtion\n' + \
          ('kB = {0:}, NB = {1:}, kC = {2:}, NC = {3:}\n'.format(kB, NB, kC, NC)) +\
          ('kBC = {0:}, NBC = {1:}'.format(kBC, NBC)), fontsize = 16)
plt.legend()
plt.tight_layout()
plt.savefig('FigPd.png')
plt.show()
